@@ -431,6 +431,15 @@ class Literal(Node):
     def __deepcopy__(self, memo=None):
         return self
 
+    def __hash__(self):
+        return hash(self.value)
+
+    def __eq__(self, other):
+        if isinstance(other, Literal):
+            return self.value == other.value
+        else:
+            return False
+
     def name(self):
         return "Literal %s at %s" % (self.value, self.position)
 
@@ -471,6 +480,15 @@ class Number(Node):
     def __deepcopy__(self, memo=None):
         return self
 
+    def __hash__(self):
+        return hash(self.value)
+
+    def __eq__(self, other):
+        if isinstance(other, Number):
+            return self.value == other.value
+        else:
+            return False
+
     def name(self):
         return "Number %f at %s" % (self.value, self.position)
 
@@ -494,6 +512,15 @@ class Constant(Node):
     def __deepcopy__(self, memo=None):
         return self
 
+    def __hash__(self):
+        return hash(self.value)
+
+    def __eq__(self, other):
+        if isinstance(other, Constant):
+            return self.value == other.value
+        else:
+            return False
+
     def name(self):
         return "Constant %s at %s" % (self.value, self.position)
 
@@ -524,6 +551,9 @@ class Undefined(Node):
     def __contains__(self, o):
         return False
 
+    def __hash__(self):
+        return hash(None)
+
     def __eq__(self, other):
         "Only equal to undefined"
         return isinstance(other, (Undefined, Empty))
@@ -553,6 +583,9 @@ class Empty(Node):
         "Acts like False"
         return False
 
+    def __hash__(self):
+        return hash(None)
+
     def __eq__(self, other):
         "Only equal to things of zero length"
         if isinstance(other, (Empty, Undefined)):
@@ -726,3 +759,77 @@ class CachedNode(Node):
         if hasattr(self.expr, "failure_info"):
             self.expr.failure_info(ctx)
 
+
+class LiteralSet(Node):
+    """
+    This node represents a 'set' of values. It is constructed
+    as a literal in the predicate and is used for things like
+    contains.
+    """
+    static = False
+    static_val = None
+
+    def __init__(self, value):
+        self.value = frozenset(value)
+
+    def name(self):
+        return "Set of %s" % repr(self.value)
+
+    def static_resolve(self, pred):
+        "Uses the predicate to perform a static resolution"
+        static = True
+        static_val = set([])
+
+        # Check if each item is static
+        for item in self.value:
+            # Only check literal, since Number and Constant
+            # are always static
+            if isinstance(item, Literal):
+                item.static_resolve(pred)
+                if item.static:
+                    static_val.add(item.static_val)
+                else:
+                    static = False
+            else:
+                static_val.add(item.value)
+
+
+        # If all items are static, so are we
+        if static:
+            self.static = True
+            self.static_val = static_val
+
+    def __nonzero__(self):
+        "Acts like False"
+        return len(self.value) > 0
+
+    def __contains__(self, o):
+        return o in self.value
+
+    def __eq__(self, other):
+        if len(self.value) == 0 and isinstance(other, (Undefined, Empty)):
+            return True
+        elif isinstance(other, LiteralSet):
+            return self.value == other.value
+        elif isinstance(other, set):
+            return self.value == other
+        elif isinstance(other, list):
+            return self.value == set(other)
+        else:
+            return False
+
+    def eval(self, ctx):
+        # Static sets are easy
+        if self.static:
+            return self.static_val
+
+        # Non-static set requires resolution of each literal value
+        s = set([])
+        for i in self.value:
+            if isinstance(i, Literal):
+                v = i.eval(ctx)
+            else:
+                v = i.value
+            s.add(v)
+        return frozenset(s)
+
